<?php

/**
 * @file
 * Administrative page for handling updates from one Drupal version to another.
 *
 * Copy this file to the root directory of your drupal installation.
 * Login as user 1, or disable the access checking below
 * Point your browser to "http://www.site.com/node_privacy_byrole_upgrade.php" and follow the
 * instructions.
 *
 * If you are not logged in as administrator, you will need to modify the access
 * check statement below. Change the TRUE into a FALSE to disable the access
 * check. After finishing the upgrade, be sure to open this file and change the
 * FALSE back into a TRUE!
 */

include_once "includes/bootstrap.inc";
include_once "includes/common.inc";
 
// Enable access checking?
$access_check = TRUE;

$sql_updates = array(0  => '',
                     1  => 'Update for Drupal 4.5 => Drupal 4.6',
                     2  => '1st update AFTER Drupal 4.5 => Drupal 4.6 update'
                    );

if (!ini_get("safe_mode")) {
  set_time_limit(180);
}

function node_privacy_byrole_upgrade_page_header($title) {
  $output = "<html><head><title>$title</title>";
  $output .= <<<EOF
      <link rel="stylesheet" type="text/css" media="print" href="misc/print.css" />
      <style type="text/css" title="layout" media="Screen">
        @import url("misc/drupal.css");
      </style>
EOF;
  $output .= "</head><body>";
  $output .= "<div id=\"logo\"><a href=\"http://drupal.org/\"><img src=\"misc/druplicon-small.png\" alt=\"Druplicon - Drupal logo\" title=\"Druplicon - Drupal logo\" /></a></div>";
  $output .= "<div id=\"update\"><h1>$title</h1>";
  return $output;
}

function node_privacy_byrole_upgrade_page_footer() {
  return "</div></body></html>";
}

function node_privacy_byrole_upgrade_page() {
  global $user, $sql_updates;

  if (isset($_POST['edit'])) {
    $edit = $_POST['edit'];
  }
  if (isset($_POST['op'])) {
    $op = $_POST['op'];
  }

  switch ($op) {
    case "Update":
      $edit = $_POST['edit'];

      if (!isset($edit['npbr_update_version']) || $edit['npbr_update_version'] == 0) { 
      
      //*****************************
        $form = form_select('Select the update after your most recent update', 'npbr_update_version', '', $sql_updates, 'Please select what you think was the last update for this module.  If you\'re not sure of what your last update was, it is better to select one that is older than what you think than to select one that is newer.  Updates that are too old for you will fail while the newer ones will still execute.');
        $form .= form_submit('Update');
        $form = form($form);
        print node_privacy_byrole_upgrade_page_header('node_privacy_byrole.module database update');
        print node_privacy_byrole_upgrade_info();
        print '<font color="#FF0000">You need to select your last update first!</font>'."<br/>\n";
        print $form;
        print node_privacy_byrole_upgrade_page_footer();
        break;
      //*****************************
      }
      else {
        // make sure we have updates to run.
        print node_privacy_byrole_upgrade_page_header("node_privacy_byrole.module database update");
        $links[] = "<a href=\"index.php\">main page</a>";
        $links[] = "<a href=\"index.php?q=admin\">administration pages</a>";
        print theme("item_list", $links);
      
        node_privacy_byrole_upgrade($edit['npbr_update_version']);

        print node_privacy_byrole_upgrade_page_footer();
      }
      break;
    default:
      $form = form_select('Select the update after your most recent update.', 'npbr_update_version', '', array_merge(array('' => ''), $sql_updates), 'Please select what you think was the last update for this module.  If you\'re not sure of what your last update was, it is better to select one that is older than what you think than to select one that is newer.  Updates that are too old for you will fail while the newer ones will still execute.');
      $form .= form_submit('Update');
      $form = form($form);
      print node_privacy_byrole_upgrade_page_header('node_privacy_byrole.module database update');
      print node_privacy_byrole_upgrade_info();
      print $form;
      print node_privacy_byrole_upgrade_page_footer();
      break;
  }
}

function node_privacy_byrole_upgrade($update_n) {
  global $base_url, $sql_updates;
  
  $enabled = module_exist('node_privacy_byrole');
  $enabled_again = variable_get('node_privacy_byrole_enabled', FALSE);
  if ($enabled && !$enabled_again) {
    print '<p>It appears that you are trying to use node_privacy_byrole, but haven\'t enabled it properly.  Because this module requires special tasks to occur upon enabling/disabling the module, and because Drupal does not automatically do it for you, you must enable the module a second time on the <a href="'. $base_url .'/?q=admin/settings/node_privacy_byrole">node_privacy_byrole\'s settings page</a>.  Please make sure to do this when you go back to your site.</p>'."\n";
  }
  
  $update_keys = array_keys($sql_updates);
  $number = $update_n;
  while (in_array($number, $update_keys)) {
    print "PERFORMING UPDATE: <br/>\n";
    print '<pre>  '. $sql_updates[$number] ."</pre><br/>\n<br/>\n";
    $function = 'node_privacy_byrole_update_'. $number;
    $function();
    $number++;
  }
  
  print '<p>Your node_privacy_byrole permissions system has been repaired and updated.</p>'."\n";
  print '<p>You are strongly advised to make sure you upgrade to at least 4.6</p>'."\n";
  
  
}

function node_privacy_byrole_upgrade_info() {
  $output = "<ul>\n";
  $output .= "<li>Use this script to <strong>fix and update permissions you've set with node_privacy_byrole</strong>.  You don't need this script when installing the node_privacy_byrole.module from scratch.</li>"."\n";
  $output .= "<li>In versions of node_privacy_byrole prior to 4.6, there was an issue with users not being able to find and see in listing the nodes that they didn't have permissions to see, but still being able to go directly to the URL of an off-limits node and view it.  This database upgrade is a way of correcting that problem.</li>"."\n";
  $output .= "<li>There is also the first update after the initial 4.6 release for this module that corrects a bug preventing node_privacy_byrole\'s ability to work with the flexinode module.</li>\n";
  $output .= "</ul>"."\n";
  return $output;
}

if (isset($_GET["op"])) {
  // Access check:
  if (($access_check == 0) || ($user->uid == 1)) {
    node_privacy_byrole_upgrade_page();
  }
  else {
    print node_privacy_byrole_upgrade_page_header("Access denied");
    print "Access denied.  You are not authorized to access to this page.  Please log in as the user with user ID #1. If you cannot log-in, you will have to edit <code>modules/node_privacy_byrole/node_privacy_byrole_upgrade.php</code> to by-pass this access check; in that case, open <code>modules/node_privacy_byrole/node_privacy_byrole_upgrade.php</code> in a text editor and follow the instructions at the top.";
    print node_privacy_byrole_upgrade_page_footer();
  }
}
else {
  //node_privacy_byrole_upgrade_info();
  node_privacy_byrole_upgrade_page();
}

function node_privacy_byrole_update_1() {
  $roles = array_keys(user_roles());
  
  $result = db_query('SELECT nid, uid FROM {node}');
  
  while ($row = db_fetch_object($result)) {
    foreach ($roles as $rid) {
      $res = db_query("SELECT nid FROM {node_access} WHERE nid = %d AND realm = '%s' AND gid = %d", $row->nid, 'node_privacy_byrole_role', $rid);
      
      if (!db_fetch_object($res)) {
        db_query("INSERT INTO {node_access} (nid, gid, realm, grant_view, grant_update, grant_delete) VALUES (%d, %d, '%s', %d, %d, %d)", $row->nid, $rid, 'node_privacy_byrole_role', 0, 0, 0);
      }
    }
    if ($row->nid == 0) {
      db_query("DELETE FROM {node_access} WHERE nid = %d AND realm = '%s' AND gid = %d", $row->nid, 'node_privacy_byrole_user', 0);
    }
  }
}

function node_privacy_byrole_update_2() {
  $result = db_query("SELECT * FROM {variable} WHERE name LIKE 'node_privacy_byrole_forview%_default_%' OR name LIKE 'node_privacy_byrole_foredit%_default_%'");
  while ($row = db_fetch_object($result)) {
    $node_type = npbr_upgrade_get_old_type($row->name);
    $rid = npbr_upgrade_get_old_rid($row->name);
    $permtype = npbr_upgrade_get_old_permtype($row->name);
    $newvarname = 'npbr_'. $permtype .'_'. $node_type .'_default_'. $rid;
    db_query("INSERT INTO {variable} (name, value) VALUES ('%s', '%s')", $newvarname, $row->value);
    // db_query("DELETE FROM {variable} WHERE name = '%s'", $row->name);
    // Refrain from deleting old values in case there is a mix up in the upgrade process.
  }
  
  $result = db_query("SELECT * FROM {variable} WHERE name LIKE 'node_privacy_byrole_metaperm_%'");
  while ($row = db_fetch_object($result)) {
    $node_type = npbr_upgrade_get_old_metapermtype($row->name);
    $rid = npbr_upgrade_get_old_rid($row->name);
    $newvarname = 'npbr_metaperm_'. $node_type .'_'. $rid;
    db_query("INSERT INTO {variable} (name, value) VALUES ('%s', '%s')", $newvarname, $row->value);
  }
  // db_query("DELETE FROM {variable} WHERE name = '%s'", $row->name);
  // Refrain from deleting old values in case there is a mix up in the upgrade process.
}

function npbr_upgrade_get_old_type($string) {
  $index = strpos($string, '_default_');
  $length = $index - 28;
  return substr($string, 28, $length);
}

function npbr_upgrade_get_old_rid($string) {
  $matches = array();
  preg_match("/.*_([0-9]+)$/i", $string, $matches);
  return $matches[1];
}

// "foredit" vs "forview"
function npbr_upgrade_get_old_permtype($string) {
  return substr($string, 20, 7);
}


function npbr_upgrade_get_old_metapermtype($string) {
  $rid = npbr_upgrade_get_old_rid($string);
  $rid = '_'. $rid;
  $ridindex = strpos($string, $rid);
  return substr($string, 29, $ridindex - 29);
}

?>